@@ -1,8 +1,9 @@
+# $Id: Compare.pm,v 1.9 2008/08/20 13:55:52 drhyde Exp $
 # Data::Compare - compare perl data structures
 # Author: Fabien Tassin <fta@sofaraway.org>
 # updated by David Cantrell <david@cantrell.org.uk>
 # Copyright 1999-2001 Fabien Tassin <fta@sofaraway.org>
-# portions Copyright 2003 - 2007 David Cantrell
+# portions Copyright 2003 - 2008 David Cantrell
 
 package Data::Compare;
 
@@ -12,22 +13,27 @@ use warnings;
 use vars qw(@ISA @EXPORT $VERSION $DEBUG %been_there);
 use Exporter;
 use Carp;
+use Scalar::Util;
 
 @ISA     = qw(Exporter);
 @EXPORT  = qw(Compare);
-$VERSION = '0.17';
+$VERSION = 1.200_500;
 $DEBUG   = 0;
 
 my %handler;
 
 use Cwd;
-if(eval { chdir(getcwd()) }) { # chdir(getcwd()) is Bad in taint mode
-    use File::Find::Rule;
-    _register_plugins();
+
+sub import {
+    if(eval { chdir(getcwd()) }) { # chdir(getcwd()) isn't taint-safe
+        register_plugins();
+    }
+    __PACKAGE__->export_to_level(1, @EXPORT);
 }
 
 # finds and registers plugins
-sub _register_plugins {
+sub register_plugins {
+    eval 'use File::Find::Rule';
     foreach my $file (
         File::Find::Rule
             ->file()
@@ -73,8 +79,6 @@ sub _register_plugins {
     }
 }
 
-sub Compare ($$;$);
-
 sub new {
   my $this = shift;
   my $class = ref($this) || $this;
@@ -85,7 +89,7 @@ sub new {
   return $self;
 }
 
-sub Cmp ($;$$) {
+sub Cmp {
   my $self = shift;
 
   croak "Usage: DataCompareObj->Cmp(x, y)" unless $#_ == 1 || $#_ == -1;
@@ -95,12 +99,16 @@ sub Cmp ($;$$) {
   return Compare($x, $y);
 }
 
-sub Compare ($$;$) {
+sub Compare {
     croak "Usage: Data::Compare::Compare(x, y, [opts])\n" unless $#_ == 1 || $#_ == 2;
 
-    my $x = shift @_;
-    my $y = shift @_;
-    my $opts = (shift @_) || {};
+    my $x = shift;
+    my $y = shift;
+    my $opts = shift || {};
+    my($xparent, $xpos, $yparent, $ypos) = map {
+        $opts->{$_} || ''
+    } qw(xparent xpos yparent ypos);
+
     my $rval = '';
 
     if(!exists($opts->{recursion_detector})) {
@@ -112,13 +120,13 @@ sub Compare ($$;$) {
     warn "Yaroo! deep recursion!\n" if($opts->{recursion_detector} == 99);
   
     if(
-        (ref($x) && exists($been_there{$x}) && $been_there{$x} > 1) ||
-        (ref($y) && exists($been_there{$y}) && $been_there{$y} > 1)
+        (ref($x) && exists($been_there{"$x-$xpos-$xparent"}) && $been_there{"$x-$xpos-$xparent"} > 1) ||
+        (ref($y) && exists($been_there{"$y-$ypos-$yparent"}) && $been_there{"$y-$ypos-$yparent"} > 1)
     ) {
         $rval = 0; # is this the right thing to do?
     } else {
-        $been_there{$x}++ if(ref($x));
-        $been_there{$y}++ if(ref($y));
+        $been_there{"$x-$xpos-$xparent"}++ if(ref($x));
+        $been_there{"$y-$ypos-$yparent"}++ if(ref($y));
 
         $opts->{ignore_hash_keys} = { map {
             ($_, 1)
@@ -145,15 +153,15 @@ sub Compare ($$;$) {
             $rval = 1;
         }
         elsif ($refx eq 'SCALAR' || $refx eq 'REF') {
-            $rval = Compare($$x, $$y, $opts);
+            $rval = Compare(${$x}, ${$y}, $opts);
         }
         elsif ($refx eq 'ARRAY') {
-            if ($#$x == $#$y) { # same length
+            if ($#{$x} == $#{$y}) { # same length
                 my $i = -1;
                 $rval = 1;
                 for (@$x) {
             	    $i++;
-      	            $rval = 0 unless Compare($$x[$i], $$y[$i], $opts);
+      	            $rval = 0 unless Compare($x->[$i], $y->[$i], { %{$opts}, xparent => $x, xpos => $i, yparent => $y, ypos => $i});
                 }
             }
             else {
@@ -167,8 +175,8 @@ sub Compare ($$;$) {
             $rval = 0 unless scalar @kx == scalar @ky;
 
             for (@kx) {
-                next unless defined $$x{$_} || defined $$y{$_};
-                $rval = 0 unless defined $$y{$_} && Compare($$x{$_}, $$y{$_}, $opts);
+                next unless defined $x->{$_} || defined $y->{$_};
+                $rval = 0 unless defined $y->{$_} && Compare($x->{$_}, $y->{$_}, { %{$opts}, xparent => $x, xpos => $_, yparent => $y, ypos => $_});
             }
         }
         elsif($refx eq 'Regexp') {
@@ -185,20 +193,20 @@ sub Compare ($$;$) {
             if ($type eq 'HASH') {
                 my %x = %$x;
                 my %y = %$y;
-                $rval = Compare(\%x, \%y, $opts);
-                $been_there{\%x}--; # decrement count for temp structures
-                $been_there{\%y}--;
+                $rval = Compare(\%x, \%y, { %{$opts}, xparent => $xparent, xpos => $xpos, yparent => $yparent, ypos => $ypos});
+                $been_there{\%x."-$xpos-$xparent"}--; # decrement count for temp structures
+                $been_there{\%y."-$ypos-$yparent"}--;
             }
             elsif ($type eq 'ARRAY') {
                 my @x = @$x;
                 my @y = @$y;
-                $rval = Compare(\@x, \@y, $opts);
-                $been_there{\@x}--;
-                $been_there{\@y}--;
+                $rval = Compare(\@x, \@y, { %{$opts}, xparent => $xparent, xpos => $xpos, yparent => $yparent, ypos => $ypos});
+                $been_there{\@x."-$xpos-$xparent"}--;
+                $been_there{\@y."-$ypos-$yparent"}--;
             }
             elsif ($type eq 'SCALAR' || $type eq 'REF') {
-                my $x = $$x;
-                my $y = $$y;
+                my $x = ${$x};
+                my $y = ${$y};
                 $rval = Compare($x, $y, $opts);
                 # $been_there{\$x}--;
                 # $been_there{\$y}--;
@@ -345,7 +353,11 @@ The module takes plug-ins so you can provide specialised routines for
 comparing your own objects and data-types.  For details see
 L<Data::Compare::Plugins>.
 
-Plugins are *not* available when running in "taint" mode.
+Plugins are *not* available when running in "taint" mode.  You may
+also make it not load plugins by providing an empty list as the
+argument to import() - ie, by doing this:
+
+    use Data::Compare ();
 
 A couple of functions are provided to examine what goodies have been
 made available through plugins:
@@ -365,6 +377,22 @@ Returns formatted text
 
 =back
 
+=head1 EXPORTS
+
+For historical reasons, the Compare() function is exported.  If you
+don't want this, then pass an empty list to import() as explained
+under PLUGINS.  If you want no export but do want plugins, then pass
+the empty list, and then call the register_plugins class method:
+
+    use Data::Compare ();
+    Data::Compare->register_plugins;
+
+or you could call it as a function if that floats your boat.
+
+=head1 CVS
+
+L<http://drhyde.cvs.sourceforge.net/drhyde/perlmodules/Data-Compare/>
+
 =head1 BUGS
 
 Plugin support is not quite finished (see the TODO file for details) but
@@ -388,7 +416,7 @@ Copyright (c) 1999-2001 Fabien Tassin. All rights reserved.
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
-Some parts copyright 2003 - 2007 David Cantrell.
+Some parts copyright 2003 - 2008 David Cantrell.
 
 Seeing that Fabien seems to have disappeared, David Cantrell has become
 a co-maintainer so he can apply needed patches.  The licence, of course,
